package Peace::DAO::Application;

use v5.30.0;

use strict;
use warnings;

use Params::ValidationCompiler qw/validation_for/;
use Types::Standard qw/HasMethods InstanceOf Str/;

{
    my $validator = validation_for(
        params => {
            dbh => { type => HasMethods ['selectrow_hashref'], optional => 1 },
        }
    );

    sub new {
        my $class  = shift;
        my %params = $validator->(@_);

        my $self = bless {}, $class;
        $self->{dbh} = $params{dbh};
        return $self;
    }
}

{
    my $validator = validation_for(
        params => {
            application => { type => InstanceOf ['Peace::Model::Application'] },
        }
    );

    sub create {
        my $self        = shift;
        my %params      = $validator->(@_);
        my $application = $params{application};
        my $dbh         = $self->_dbh;

        my $insert = <<'EOF';
INSERT INTO applications
    (name, description, url,
    developer, price, git_repo,
    flatpak_builder_file, verified)
VALUES (?, ?, ?, ?, ?, ?, ?, ?)
RETURNING uuid;
EOF
        my $result = $dbh->selectrow_hashref(
            $insert,                            undef,
            $application->name,                 $application->description,
            $application->url,                  $application->developer->uuid,
            $application->price,                $application->git_repo,
            $application->flatpak_builder_file, $application->verified
        );
        my $uuid            = $result->{uuid};
        my $new_application = $self->recover_by_uuid( uuid => $uuid );

        $application->uuid( $new_application->uuid );
        $application->date_creation( $new_application->date_creation );

        return $application;
    }
}

{
    my $validator = validation_for(
        params => {
            uuid => { type => Str },
        }
    );

    sub recover_by_uuid {
        my $self   = shift;
        my %params = $validator->(@_);
        my $dbh    = $self->_dbh;
        my $uuid   = $params{uuid};
        my $result = $dbh->selectrow_hashref( <<'EOF', undef, $uuid );
SELECT uuid, date_creation, name,
    description, url, developer,
    price, git_repo, flatpak_builder_file,
    verified
FROM applications
WHERE uuid = ?
EOF
        if ( !defined $result ) {
            die "Application with uuid $uuid not found.";
        }
        if ( exists $result->{developer} ) {
            $result->{developer_uuid} = delete $result->{developer};
            $result->{dbh} = $self->_dbh;
        }
        if ( exists $result->{date_creation} ) {
            my $iso8601 = DateTime::Format::Pg->new;
            $result->{date_creation} =
              $iso8601->parse_datetime( $result->{date_creation} );
        }
        my $application = Peace::Model::Application->new(%$result);
        return $application;
    }
}

sub _dbh {
    my $self = shift;
    return $self->{dbh};
}
1;
